@@ -2,17 +2,18 @@
 
 from __future__ import (absolute_import, division, print_function)
 
+from math import log
 
 import numpy as np
 
 
-def plot_result(x, y, params=(), indices=None, plot=None, plot_kwargs_cb=None,
+def plot_result(x, y, indices=None, plot=None, plot_kwargs_cb=None, ax=None,
                 ls=('-', '--', ':', '-.'),
                 c=('k', 'r', 'g', 'b', 'c', 'm', 'y'),
                 m=('o', 'v', '8', 's', 'p', 'x', '+', 'd', 's'),
                 m_lim=-1, lines=None, interpolate=None, interp_from_deriv=None,
-                names=None, post_processors=(), xlabel=None, ylabel=None,
-                xscale=None, yscale=None, latex_attr='latex_name'):
+                names=None, latex_names=None, xlabel=None, ylabel=None,
+                xscale=None, yscale=None):
     """
     Plot the depepndent variables vs. the independent variable
 
@@ -25,14 +26,13 @@ def plot_result(x, y, params=(), indices=None, plot=None, plot_kwargs_cb=None,
         ``y.shape[0] == len(x)``, plot_results will draw
         ``y.shape[1]`` lines. If ``interpolate != None``
         y is expected two be three dimensional, otherwise two dimensional.
-    params : array_like
-        Parameters used.
     indices : iterable of integers
         What indices to plot (default: None => all).
     plot : callback (default: None)
         If None, use ``matplotlib.pyplot.plot``.
     plot_kwargs_cb : callback(int) -> dict
         Keyword arguments for plot for each index (0:len(y)-1).
+    ax : Axes
     ls : iterable
         Linestyles to cycle through (only used if plot and plot_kwargs_cb
         are both None).
@@ -52,64 +52,59 @@ def plot_result(x, y, params=(), indices=None, plot=None, plot_kwargs_cb=None,
         if True => 20. negative integer signifies log-spaced grid.
     interp_from_deriv : callback (default: None)
         When ``None``: ``scipy.interpolate.BPoly.from_derivatives``
-    post_processors : iterable of callback (default: tuple())
 
     """
     import matplotlib.pyplot as plt
 
     if plot is None:
-        from matplotlib.pyplot import plot
+        if ax is None:
+            from matplotlib.pyplot import plot
+        else:
+            plot = ax.plot
     if plot_kwargs_cb is None:
         def plot_kwargs_cb(idx, lines=False, markers=False, labels=None):
-            kwargs = {'c': c[idx % len(c)]}
+
+            kw = {'c': c[idx % len(c)]}
 
             if lines:
-                kwargs['ls'] = ls[idx % len(ls)]
+                kw['ls'] = ls[idx % len(ls)]
                 if isinstance(lines, float):
-                    kwargs['alpha'] = lines
+                    kw['alpha'] = lines
             else:
-                kwargs['ls'] = 'None'
+                kw['ls'] = 'None'
 
             if markers:
-                kwargs['marker'] = m[idx % len(m)]
+                kw['marker'] = m[idx % len(m)]
 
             if labels:
-                kwargs['label'] = labels[idx]
-                if latex_attr:
-                    try:
-                        kwargs['label'] = '$' + getattr(labels[idx],
-                                                        latex_attr) + '$'
-                    except (AttributeError, TypeError):
-                        pass
-            return kwargs
+                kw['label'] = labels[idx]
+            return kw
     else:
         plot_kwargs_cb = plot_kwargs_cb or (lambda idx: {})
 
-    def post_process(x, y, params):
-        for post_processor in post_processors:
-            x, y, params = post_processor(x, y, params)
-        return x, y, params
-
     if interpolate is None:
         interpolate = y.ndim == 3 and y.shape[1] > 1
 
-    x_post, y_post, params_post = post_process(x, y[:, 0, :] if interpolate and
-                                               y.ndim == 3 else y, params)
+    if interpolate and y.ndim == 3:
+        _y = y[:, 0, :]
+    else:
+        _y = y
+
     if indices is None:
-        indices = range(y_post.shape[-1])  # e.g. PartiallySolvedSys
+        indices = range(_y.shape[-1])  # e.g. PartiallySolvedSys
     if lines is None:
         lines = interpolate in (None, False)
     markers = len(x) < m_lim
     for idx in indices:
-        plot(x_post, y_post[:, idx], **plot_kwargs_cb(
-            idx, lines=lines, labels=names))
+        plot(x, _y[:, idx], **plot_kwargs_cb(
+            idx, lines=lines, labels=latex_names or names))
         if markers:
-            plot(x_post, y_post[:, idx], **plot_kwargs_cb(
-                idx, lines=False, markers=markers, labels=names))
+            plot(x, _y[:, idx], **plot_kwargs_cb(
+                idx, lines=False, markers=markers, labels=latex_names or names))
 
     if xlabel is None:
         try:
-            plt.xlabel(x_post.dimensionality.latex)
+            plt.xlabel(x.dimensionality.latex)
         except AttributeError:
             pass
     else:
@@ -117,7 +112,7 @@ def plot_result(x, y, params=(), indices=None, plot=None, plot_kwargs_cb=None,
 
     if ylabel is None:
         try:
-            plt.ylabel(y_post.dimensionality.latex)
+            plt.ylabel(_y.dimensionality.latex)
         except AttributeError:
             pass
     else:
@@ -144,46 +139,39 @@ def plot_result(x, y, params=(), indices=None, plot=None, plot_kwargs_cb=None,
             import scipy.interpolate
             interp_from_deriv = scipy.interpolate.BPoly.from_derivatives
 
-        y2 = np.empty((x_plot.size, y.shape[-1]))
-        for idx in range(y.shape[-1]):
+        y2 = np.empty((x_plot.size, _y.shape[-1]))
+        for idx in range(_y.shape[-1]):
             interp_cb = interp_from_deriv(x, y[..., idx])
             y2[:, idx] = interp_cb(x_plot)
 
-        x_post2, y_post2, params2 = post_process(x_plot, y2, params)
         for idx in indices:
-            plot(x_post2, y_post2[:, idx], **plot_kwargs_cb(
+            plot(x_plot, y2[:, idx], **plot_kwargs_cb(
                 idx, lines=True, markers=False))
-        return x_post2, y_post2
+        return x_plot, y2
 
     if xscale is not None:
-        plt.gca().set_xscale(xscale)
+        (ax or plt.gca()).set_xscale(xscale)
     if yscale is not None:
-        plt.gca().set_yscale(yscale)
-    return x_post, y_post
+        (ax or plt.gca()).set_yscale(yscale)
 
 
-def plot_phase_plane(x, y, params=(), indices=None, post_processors=(),
-                     plot=None, names=None, **kwargs):
+def plot_phase_plane(x, y, indices=None, plot=None, names=None, **kwargs):
     """ Plot the phase portrait of two dependent variables
 
     Parameters
     ----------
     x: array_like
-        Values of the independent variable
+        Values of the independent variable.
     y: array_like
-        Values of the dependent variables
-    params: array_like
-        parameters
+        Values of the dependent variables.
     indices: pair of integers (default: None)
-        what dependent variable to plot for (None => (0, 1))
-    post_processors: iterable of callbles
-        see :class:OdeSystem
+        What dependent variable to plot for (None => (0, 1)).
     plot: callable (default: None)
-        Uses matplotlib.pyplot.plot if None
+        Uses ``matplotlib.pyplot.plot`` if ``None``
     names: iterable of strings
-        labels for x and y axis
+        Labels for x and y axis.
     \*\*kwargs:
-        keyword arguemtns passed to ``plot()``
+        Keyword arguemtns passed to ``plot()``.
 
     """
     if indices is None:
@@ -198,7 +186,57 @@ def plot_phase_plane(x, y, params=(), indices=None, post_processors=(),
             plt.xlabel(names[indices[0]])
             plt.ylabel(names[indices[1]])
 
-    for post_processor in post_processors:
-        x, y, params = post_processor(x, y, params)
-
     plot(y[:, indices[0]], y[:, indices[1]], **kwargs)
+
+
+def right_hand_ylabels(ax, labels):
+    ax2 = ax.twinx()
+    ylim = ax.get_ylim()
+    yspan = ylim[1]-ylim[0]
+    ax2.set_ylim(ylim)
+    yticks = [ylim[0] + (idx + 0.5)*yspan/len(labels) for idx in range(len(labels))]
+    ax2.tick_params(length=0)
+    ax2.set_yticks(yticks)
+    ax2.set_yticklabels(labels)
+
+
+def info_vlines(ax, xout, info, vline_colors=('maroon', 'purple'),
+                vline_keys=('steps', 'rhs_xvals', 'jac_xvals'),
+                post_proc=None, alpha=None, fpes=None, every=None):
+    """ Plot vertical lines in the background
+
+    Parameters
+    ----------
+    ax : axes
+    xout : array_like
+    info : dict
+    vline_colors : iterable of str
+    vline_keys : iterable of str
+        Choose from ``'steps', 'rhs_xvals', 'jac_xvals',
+        'fe_underflow', 'fe_overflow', 'fe_invalid', 'fe_divbyzero'``.
+    vline_post_proc : callable
+    alpha : float
+
+    """
+
+    nvk = len(vline_keys)
+    for idx, key in enumerate(vline_keys):
+        if key == 'steps':
+            vlines = xout
+        elif key.startswith('fe_'):
+            if fpes is None:
+                raise ValueError("Need fpes when vline_keys contain fe_*")
+            vlines = xout[info['fpes'] & fpes[key.upper()] > 0]
+        else:
+            vlines = info[key] if post_proc is None else post_proc(info[key])
+
+        if alpha is None:
+            alpha = 0.01 + 1/log(len(vlines)+3)
+
+        if every is None:
+            ln_np1 = log(len(vlines)+1)
+            every = min(round((ln_np1 - 4)/log(2)), 1)
+
+        ax.vlines(vlines[::every], idx/nvk + 0.002, (idx+1)/nvk - 0.002, colors=vline_colors[idx % len(vline_colors)],
+                  alpha=alpha, transform=ax.get_xaxis_transform())
+    right_hand_ylabels(ax, [k[3] if k.startswith('fe_') else k[0] for k in vline_keys])
